/* player structure:
   /id  -> RINCON_B8E937809DD601400
   /serial -> Mac
   /typename  -> Sonos PLAY:1
   /deviceName -> Wohnzimmer
   /zoneID  -> DS name
   /disabled
   /online
   /ip
    */

Property.setSavedProperty=function (path,value) {
	  Property.setProperty(path, value);
	  Property.setFlag(path, 'ARCHIVE', true);
}


function getFirstActiveIP() {
	var oPlayerArray = Property.getNode('players').getChildren();
	for (var iIndex = 0; iIndex < oPlayerArray.length; iIndex++) {
		if (oPlayerArray[iIndex].getChild('online').getValue()==true)
			if (oPlayerArray[iIndex].getChild('invisible').getValue()==false)
				return oPlayerArray[iIndex].getChild('ip').getValue();
	}
	return null;
}

var LOGFILE_NAME = "sonos-controller.log";
var lWarn = {
		logln : function(o) {
		}
	};
	var lInfo = {
		logln : function(o) {
		}
	};

lInfo = new Logger(LOGFILE_NAME);
lWarn = new Logger(LOGFILE_NAME);

function updatePlayer(oneIPAdress) {
	lInfo.logln('updatePlayer');
	try {
		
		var h = new HTTP({'debug' : 0,'connectTimeout' : 200}, {});
		lInfo.logln('Fetch infos from '+ oneIPAdress);
		var answer=upnpGetSonosGroups(oneIPAdress,true);
		if (answer==null){
			lWarn.logln('Info is null ...');
			return false;
		}

		if (Property.getNode('players')!=null) {
			var playersArray=Property.getNode('players').getChildren();
			for (var iIndex=0;iIndex<playersArray.length;iIndex++) {
				var sNodePath=playersArray[iIndex].getPath();
				Property.setSavedProperty(sNodePath + '/online',false);
			}
		}

		
		lInfo.logln(JSON.stringify(answer));	
		for (var iIndex=0;iIndex<answer.length;iIndex++) {
			for (var jIndex=0;jIndex<answer[iIndex].member.length;jIndex++) {
				var id=answer[iIndex].member[jIndex].id;			
				lInfo.logln('fetch ' + answer[iIndex].member[jIndex].deviceXML);
				var answ=h.get(answer[iIndex].member[jIndex].deviceXML);
				if (answ==null) {
					lInfo.logln('freshly lost Player !!!');
				} else {
					Property.setSavedProperty('players/'+id+'/id',id);
					Property.setSavedProperty('players/'+id+'/serial',getXMLItem(answ.body,'serialNum').replace(':','-'));
					Property.setSavedProperty('players/'+id+'/typename',getXMLItem(answ.body,'modelName'));
					Property.setSavedProperty('players/'+id+'/deviceName',answer[iIndex].member[jIndex].name);			
					Property.setSavedProperty('players/'+id+'/ip',answer[iIndex].member[jIndex].ip);
					Property.setSavedProperty('players/'+id+'/group',answer[iIndex].id);
					Property.setSavedProperty('players/'+id+'/invisible',answer[iIndex].member[jIndex].invisible);
					Property.setSavedProperty('players/'+id+'/coordinator',(answer[iIndex].coordinator==id));				
					Property.setSavedProperty('players/'+id+'/online',true);
					if (Property.getNode('players/' + id + '/disabled')==null)
						Property.setSavedProperty('players/'+id+'/disabled',false);
					Property.getNode('players/'+id).removeChild('researchCount');

				}
			}
		}	
		lInfo.logln('finished, just ensure the predefined activities now.');
		new Event('sonos.info',{actions:'fetchSonosPlaylists'}).raise();
		new Event('sonos.info',{actions:'fetchSonosFavorites'}).raise();
		new Event('sonos.config',{actions:'ensurePredefinedEntries',time:'+1'}).raise();
		new Event('sonos.saved',{actions:'structureUpdate'}).raise();
		Property.store();
		updateZoneConnectedDevices();
		return true;
	} catch (e) {
		lWarn.logln('warning ' + e);
		return false;
	}
}

function checkIfPlayerOnline(oneIPAdress) {
	lInfo.logln('checkIfPlayerOnline');
	var h = new HTTP({'debug' : 0,'connectTimeout' : 200}, {});
	var sUrl = "http://" + oneIPAdress + ":1400/xml/device_description.xml";

	var result = h.get(sUrl);

	if (result != null)
		if (result.body != null) 
			if ((result.body.indexOf("schemas-upnp-org") != -1) && (result.body.indexOf("sonos.com</manufacturerURL>") != -1)) {
				return true;
			}
	return false;
}

function checkIfAnyPlayerOnline() {
	lInfo.logln('checkIfAnyPlayerOnline');
	if (Property.getNode('players')!=null) {
		var playersArray=Property.getNode('players').getChildren();
		var actualOnlinePlayers=[];
		var acutalOfflinePlayers=[];
		for (var iIndex=0;iIndex<playersArray.length;iIndex++) {
			var sNodePath=playersArray[iIndex].getPath();
			if (playersArray[iIndex].getChild('online').getValue())
				actualOnlinePlayers.push(playersArray[iIndex]);
			else
				acutalOfflinePlayers.push(playersArray[iIndex]);
			Property.setSavedProperty(sNodePath + '/online',false);
			researchCount=0;
			if (Property.getNode(sNodePath+'/researchCount')!=null)
				researchCount=Property.getProperty(sNodePath+'/researchCount');
			researchCount+=1;
			lInfo.logln('setting ' + researchCount + ' for ' + sNodePath);
			Property.setSavedProperty(sNodePath + '/researchCount',researchCount);
		}
		for (var iIndex=0;iIndex<actualOnlinePlayers.length;iIndex++) {
			var sPlayerIP=actualOnlinePlayers[iIndex].getChild('ip').getValue();
			lInfo.logln('check if player with ip ' + sPlayerIP + ' is online (was assumed as online!)');
			if (checkIfPlayerOnline(sPlayerIP)) {
				if (updatePlayer(sPlayerIP))
					return true;
			}
		}
		for (var iIndex=0;iIndex<acutalOfflinePlayers.length;iIndex++) {
			var sPlayerIP=acutalOfflinePlayers[iIndex].getChild('ip').getValue();
			lInfo.logln('check if player with ip ' + sPlayerIP + ' is online (was assumed as offline)');
			if (checkIfPlayerOnline(sPlayerIP)) {
				if (updatePlayer(sPlayerIP))
					return true;
			}
		}
	}
	return false;
}

function startBruteForceDiscovery() {
	lInfo.logln('startBruteForceDiscovery');
	if (raisedEvent.parameter.ip == null) {
		lWarn.logln('missing parameter ip');
		return;
	}
	var sIP=raisedEvent.parameter.ip;
	var aIP = sIP.split('.');
	if (aIP.length == 4)
		sIP = aIP[0] + "." + aIP[1] + "." + aIP[2] + ".";

	var evt = new Event("sonos.player", {
		actions:'stepDiscovery',
		baseIp : sIP,
		stepSize : 5,
		step : 1
	}).raise();
}

function stepDiscovery() {
	lInfo.logln('stepDiscovery');
	if (raisedEvent.parameter.baseIp == null) {
		lWarn.logln('missing parameter baseIp');
		return;
	}
	if (raisedEvent.parameter.stepSize == null) {
		lWarn.logln('missing parameter stepSize');
		return;
	}
	if (raisedEvent.parameter.step == null) {
		lWarn.logln('missing parameter step');
		return;
	}
	var sIP = raisedEvent.parameter.baseIp;
	var startIndex = parseInt(""+raisedEvent.parameter.step,10);
	var stepSize= parseInt(""+raisedEvent.parameter.stepSize,10);
	lInfo.logln('making autodiscovery for ' + sIP + ' with ' + stepSize + ' on step ' + startIndex);
	for (var iIndex=startIndex;(iIndex<(startIndex+stepSize)) && (iIndex<256);iIndex++ ) {
		var sSearchedIP = sIP + iIndex;
		if (checkIfPlayerOnline(sSearchedIP)) {
			if (updatePlayer(sSearchedIP)) {
				new Event('sonos.saved',{actions:'search',result:true}).raise();
				return;
			}
		}
	}
	
	if ((startIndex+stepSize)<256) {
		new Event('sonos.player',{
			actions:'stepDiscovery',
			baseIp:sIP,
			stepSize:stepSize,
			step:(startIndex+stepSize)
		}).raise();
	} else {
		new Event('sonos.saved',{actions:'search',result:false}).raise();
	}
}

function startAutoDiscovery() {
	lInfo.logln('startAutoDiscovery');
	if (!checkIfAnyPlayerOnline())
		startBruteForceDiscovery();
}

function setZone() {
	if (raisedEvent.parameter.id!=null)
		if (raisedEvent.parameter.zoneID!=null) 
			if (Property.getNode('players/' + raisedEvent.parameter.id)!=null) {
				if (raisedEvent.parameter.zoneID==0) {
					Property.getNode('players/' + raisedEvent.parameter.id).removeChild('zoneID');
					Property.store();
					new Event('sonos.config',{actions:'ensurePredefinedEntries'}).raise();
					return;
				}
				var playersArray=Property.getNode('players').getChildren();
				for (var iIndex=0;iIndex<playersArray.length;iIndex++) {
					if (playersArray[iIndex].getChild('zoneID')!=null)
						if (playersArray[iIndex].getChild('zoneID').getValue()==raisedEvent.parameter.zoneID) {
							new Event('sonos.saved',{actions:'resetZone'}).raise();
							return;							
						}
				}				
				Property.setSavedProperty('players/' + raisedEvent.parameter.id + '/zoneID',raisedEvent.parameter.zoneID);
				Property.store();	
				new Event('sonos.config',{actions:'ensurePredefinedEntries'}).raise();
			}
}

function enablePlayer() {
	if (raisedEvent.parameter.id!=null) {
		var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
        lWarn.logln(typeof(oIDObject));
        if (typeof(oIDObject)!='object')
            oIDObject=[oIDObject];

        for (var sKey in oIDObject) {
	        var sanityCheck = Property.getNode('players/'+oIDObject[sKey]);
	        if (sanityCheck != null) {                
	        	Property.setSavedProperty('players/' + oIDObject[sKey] + '/disabled',false);
	        }
        }
        Property.store();
	}
}

function disablePlayer() {
	if (raisedEvent.parameter.id!=null) {
		lInfo.logln('!!! disable ' + raisedEvent.parameter.id);
		var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
        lWarn.logln(typeof(oIDObject));
        if (typeof(oIDObject)!='object')
            oIDObject=[oIDObject];

        for (var sKey in oIDObject) {
	        var sanityCheck = Property.getNode('players/'+oIDObject[sKey]);
	        if (sanityCheck != null) {                
	        	Property.setSavedProperty('players/' + oIDObject[sKey] + '/disabled',true);
	        }
        }
        Property.store();
	}
}

function resetZone() {
	if (raisedEvent.parameter.id!=null) {
		var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
        lWarn.logln(typeof(oIDObject));
        if (typeof(oIDObject)!='object')
            oIDObject=[oIDObject];

        for (var sKey in oIDObject) {
	        var sanityCheck = Property.getNode('players/'+oIDObject[sKey]);
	        if (sanityCheck != null) {                
	        	Property.getNode('players').removeChild(oIDObject[sKey]);
	        }
        }
        Property.store();
	}	
}

function updateZoneConnectedDevices() {
	// connectedDevices
	var zones = getZones();
	for ( var iIndex = 0; iIndex < zones.length; iIndex++) {
		if (zones[iIndex].id != 0) {
			try {
				zones[iIndex].removeConnectedDevice(4);
			} catch (e) {
				Property.setProperty('/apartment/zones/zone' + zones[iIndex].id + '/groups/group4/connectedDevices', 0);
			}
		}
	}

	if (Property.getNode('players')!=null)	{
		var oArrayNode = Property.getNode('players').getChildren();
		for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
			var oChildNode = oArrayNode[iIndex];
			if (oChildNode.getChild('zoneID')!=null)
				if (oChildNode.getChild('online')!=null)
					if (oChildNode.getChild('disabled')!=null)
						if (oChildNode.getChild('online').getValue())
							if (!oChildNode.getChild('disabled').getValue()) {				
								var zoneID=parseInt(oChildNode.getChild('zoneID').getValue(),10);
								lInfo.logln("Activate connectedDevices for zoneID=" + zoneID);
								try {
									getZoneByID(zoneID).addConnectedDevice(4);
								} catch (e) {
									lInfo.logln('Failed to execute addConnectedDevice. ( '+e+') Trying to assign value manually');
									Property.setProperty('/apartment/zones/zone' + zoneID + '/groups/group4/connectedDevices', 1);
								}
							}
		}
	}
}


function searchAfterLostPlayer() {
	lInfo.logln('searchAfterLostPlayer');
	checkIfAnyPlayerOnline();
	var minResearchCount=99999;
	var playersArray=Property.getNode('players').getChildren();
	if (Property.getNode('autoSearchScheduled')!=null) {
		Property.getNode('/system/EventInterpreter/ScheduledEvents').removeChild(Property.getProperty('autoSearchScheduled'));
	}
	for (var iIndex=0;iIndex<playersArray.length;iIndex++) {
		if (playersArray[iIndex].getChild('researchCount')!=null)
			if (playersArray[iIndex].getChild('researchCount').getValue()<minResearchCount)
				minResearchCount=playersArray[iIndex].getChild('researchCount').getValue();
	}
	lInfo.logln('min researchcount ' + minResearchCount);	
	switch (minResearchCount) {
		case 1:
			lInfo.logln('player(s) are still missing, autosearch schedulded + 10 minutes ahead');
			var id=new TimedEvent('sonos.player','+600',{actions:'searchAfterLostPlayer'}).raise();
			Property.setProperty('autoSearchScheduled',id);
			break;	
		case 2:
			lInfo.logln('player(s) are still missing, autosearch schedulded + 10 minutes ahead');
			var id=new TimedEvent('sonos.player','+600',{actions:'searchAfterLostPlayer'}).raise();
			Property.setProperty('autoSearchScheduled',id);
			break;
		case 3:
			lInfo.logln('player(s) are still missing, autosearch schedulded + 60 minutes ahead');
			var id=new TimedEvent('sonos.player','+3600',{actions:'searchAfterLostPlayer'}).raise();
			Property.setProperty('autoSearchScheduled',id);
			break;
		case 4:
			lInfo.logln('player(s) are still missing, autosearch schedulded + 8 hours ahead');
			var id=new TimedEvent('sonos.player','+28800',{actions:'searchAfterLostPlayer'}).raise();
			Property.setProperty('autoSearchScheduled',id);
			break;
		default:
			lInfo.logln('player(s) might be missing, but to much retrys, so ending the autosearch');
		break;
	}
	
}

function updatePlayerDirectArray() {
	var oArray=JSON.parse(raisedEvent.parameter.referenceIPs);
	for (var iIndex=0;iIndex<oArray.length;iIndex++) {
		var resultVal=updatePlayer(oArray[iIndex]);
		if (resultVal) {
			if (raisedEvent.parameter.concurrenceEvent!=null)
				if (raisedEvent.parameter.concurrenceEventParameter!=null) {
					oParameter=JSON.parse(raisedEvent.parameter.concurrenceEventParameter);
					new Event(raisedEvent.parameter.concurrenceEvent,oParameter).raise();
				}										
			return;
		}
			
	}
}

function cleanupOldSonosPlaylists() {
	var arrayLists = upnpGetSonosPlaylists([getFirstActiveIP()]);
	lInfo.logln('ergebnis ' + arrayLists);
	var fRefetchNeeded=false;
	if (arrayLists != null)
		for (var iIndex = 0; iIndex < arrayLists.length; iIndex++) {
			if (arrayLists[iIndex].title.indexOf('dSS-tmp')==0) {
				fRefetchNeeded=true;
				upnpDeleteQueueFromList(getFirstActiveIP(),arrayLists[iIndex].listID);
			}			
		}
	if (fRefetchNeeded)
		new Event('sonos.info',{actions:'fetchSonosPlaylists'}).raise();
}


if (raisedEvent.name=='sonos.player') {
	if (raisedEvent.parameter.actions=='updatePlayerDirect') { 
		if (raisedEvent.parameter.referenceIP!=null) {
			var resultVal=updatePlayer(raisedEvent.parameter.referenceIP);
			new Event('sonos.saved',{actions:'search',result:resultVal}).raise();
		} 
		if (raisedEvent.parameter.referenceIPs!=null) {
			updatePlayerDirectArray();
			new Event('sonos.saved',{actions:'search',result:true}).raise();
		}
	}
	if (raisedEvent.parameter.actions=='searchAfterLostPlayer') {
		searchAfterLostPlayer();
	}
	if (raisedEvent.parameter.actions=='updateZoneConnectedDevices') {
		updateZoneConnectedDevices();
	}
	if (raisedEvent.parameter.actions=='updatePlayer')
		checkIfAnyPlayerOnline();
	if (raisedEvent.parameter.actions=='startBruteForceDiscovery') 
		startBruteForceDiscovery();
	if (raisedEvent.parameter.actions=='stepDiscovery') 
		stepDiscovery();
	if (raisedEvent.parameter.actions=='startAutoDiscovery')
		startAutoDiscovery();
	if (raisedEvent.parameter.actions=='setZone') {
		setZone();
		updateZoneConnectedDevices();
	}
	if (raisedEvent.parameter.actions=='enable') {
		enablePlayer();
		updateZoneConnectedDevices();
		new Event('sonos.saved',{actions:'enablePlayer'}).raise();
	}
	if (raisedEvent.parameter.actions=='disable') {
		disablePlayer();
		updateZoneConnectedDevices();
		new Event('sonos.saved',{actions:'disablePlayer'}).raise();
	}
	if (raisedEvent.parameter.actions=='resetZone') {
		resetZone();
		updateZoneConnectedDevices();
		new Event('sonos.saved',{actions:'resetZone'}).raise();
	}
	if (raisedEvent.parameter.actions=='cleanupPlaylists') {
		cleanupOldSonosPlaylists();
	}
}