
Ext.define('DSS.component.SonosDeviceLabel', {
    extend : 'DSS.component.Base',
    mixins : {
        observable : 'Ext.util.Observable'
    },
    getHeight:function () {
       return 180;
    },
    gridRooms:null,
    roomID:-1,
    roomRenderer : function(val, metadata, record, colIdx, store, view) {
       var sPassive="";        
       if (record.get('passive')!=null && record.get('passive'))
         sPassive='style="color:#888888;"';
       var sReturn='<table style="height:16px;"><tr>';
       sReturn+='<td '+sPassive+' width="100%">' + record.get('name') + '</td>';
       return sReturn +'</tr></table>';      
    },
    initStores : function() {
        // if player is disabled, user can only un-assign the room
    	var isPlayerDisabled = false;
        if (this.dssEditor.editControls.online != null)
          isPlayerDisabled = !this.dssEditor.editControls.online.getValue(); 
        
        var reservatedZones=[];       
        for (var iIndex = 0; iIndex < playerStore.getCount(); iIndex++) {
			var oRecord = playerStore.getAt(iIndex);
			if (oRecord.get('zoneID')!=null)
				if (oRecord.get('zoneID')!="")
					reservatedZones.push({zone:parseInt(''+oRecord.get('zoneID'),10),id:oRecord.get('id'),online:oRecord.get('online')});
        }
         
        var arrayRooms = new Array();                                                             
        if (dss.staticDataModel.nameCache != null) {
        	for ( var iIndex = 0; iIndex < dss.staticDataModel.nameCache.length; iIndex++) {
        		var oZone = dss.staticDataModel.nameCache[iIndex];
        		if (oZone.ZoneID > 0) {
        			var bPassive = isPlayerDisabled;		
        			for (var yIndex=0;yIndex<reservatedZones.length;yIndex++) {
        				if (reservatedZones[yIndex].zone==oZone.ZoneID) {
        					if (reservatedZones[yIndex].online)
        						bPassive = true;
    						if (this.roomID==oZone.ZoneID)
    							bPassive = isPlayerDisabled;
        				}
        			}            
		             arrayRooms.push({"_id":oZone.ZoneID,
		                              "name":oZone.name,
		                              "passive":bPassive});
           }
         } 
       } 
                                                                          
       var store = Ext.create('Ext.data.Store', {
                        storeId : 'assign_sonos_players',
                        fields : [ '_id', 'name', 'passive'],
                        proxy : {
                            type : 'memory',
                            reader : {
                                type : 'json',
                                root : 'items'
                            }
                        },
                        sorters: [{property: 'name',direction: 'ASC'}]
               });
       store.add(arrayRooms);
       // add first item at the end, so sorting of rooms is not disturbed
       store.insert(0, {"_id":0, "name":_('no Assignment'), "passive":false});
    },
    constructor : function(config) {
       var lblRoom  = Ext.create('Ext.form.field.Display',{
          fieldLabel: _('Room'),
          margin : '0 0 0 0',
        }); 

        this.gridRooms = Ext.create('Ext.grid.Panel', {
            hideHeaders:true,
            autoScroll : true,
            width:200,
            height:150,
            margin : '5 0 0 0',
            viewConfig : {
                stripeRows : true
            },
            selModel : {
                mode : 'SINGLE',
//              allowDeselect: true, 
            },
            columns : [ {
                sortable : false,
                hideable : false,
                dataIndex : '_id',
                flex : 1,
                renderer : this.roomRenderer
            } ]
        }); 
        
        this.gridRooms.on('itemclick', function(grid, record) {
            var fIsPassive=false;
            if (record.get('passive')!=null) {
                if (record.get('passive')==true) {
                    this.getSelectionModel().deselect(record);
                    fIsPassive=true;
                }
            }
        }); 
        
        var pnlRoom = Ext.create('Ext.container.Container', {
            defaultType : 'numberfield',
            items : [ lblRoom, this.gridRooms ],
            layout: {type: 'hbox'},
            margin : '5 0 5 0',
            width : 400 });
        
        Ext.apply(this, {
            items : [ pnlRoom]//, 
        });                          
        this.callParent(arguments);
    },
    reset: function (){
      this.initStores();
      this.gridRooms.reconfigure(Ext.data.StoreManager.lookup('assign_sonos_players'));
      this.roomID = -1;
    },
    setValue:function (value){
      this.roomID = parseInt("" + value,10);
      this.initStores();
      this.gridRooms.reconfigure(Ext.data.StoreManager.lookup('assign_sonos_players'));
      var store = this.gridRooms.getStore();
      var foundRec = store.getAt(store.findExact('_id', this.roomID));
      this.gridRooms.getSelectionModel().select(foundRec);
    },
    isValid:function(){
       var oSelectionRooms = this.gridRooms.getSelectionModel().getSelection();
       var bOk = (oSelectionRooms.length > 0);
       if (!bOk)
         Ext.MessageBox.alert(_('Info'), _('Please select room from list'), null);
       return bOk;
    },
    getValue : function(){
      var oSelectionRooms = this.gridRooms.getSelectionModel().getSelection();
      if (oSelectionRooms.length > 0) {
          return parseInt('' + oSelectionRooms[0].get('_id'), 10);
      }
      return null;      
    }
});